@@ -3,7 +3,7 @@ import logging
 import os
 from decimal import Decimal
 from enum import Enum
-from typing import Dict, List, Tuple, Optional, Any, Union
+from typing import Dict, List, Tuple, Optional, Any
 
 from basemodels import Manifest
 from eth_keys import keys
@@ -23,7 +23,7 @@ from hmt_escrow.eth_bridge import (
     Retry,
     HMTOKEN_ADDR,
 )
-from hmt_escrow.storage import download, upload
+from hmt_escrow.storage import download, upload, get_public_bucket_url, get_key_from_url
 
 GAS_LIMIT = int(os.getenv("GAS_LIMIT", 4712388))
 
@@ -535,6 +535,7 @@ class Job:
         results: Dict,
         pub_key: bytes,
         encrypt_final_results: bool = True,
+        store_pub_final_results: bool = False,
     ) -> bool:
         """Performs a payout to multiple ethereum addresses. When the payout happens,
         final results are uploaded to IPFS and contract's state is updated to Partial or Paid
@@ -592,8 +593,8 @@ class Job:
             payouts (List[Tuple[str, int]]): a list of tuples with ethereum addresses and amounts.
             results (Dict): the final answer results stored by the Reputation Oracle.
             pub_key (bytes): the public key of the Reputation Oracle.
-            gas (int): maximum amount of gas the caller is ready to pay.
             encrypt_final_results (bool): Whether final results must be encrypted.
+            store_pub_final_results (bool): Whether final results must be stored with public access.
 
         Returns:
             bool: returns True if paying to ethereum addresses and oracles succeeds.
@@ -608,10 +609,16 @@ class Job:
             "hmt_server_addr": self.hmt_server_addr,
         }
 
-        (hash_, url) = upload(
-            msg=results, public_key=pub_key, encrypt_data=encrypt_final_results
+        hash_, url = upload(
+            msg=results,
+            public_key=pub_key,
+            encrypt_data=encrypt_final_results,
+            use_public_bucket=store_pub_final_results,
         )
 
+        # Plain data will be publicly accessible
+        url = get_public_bucket_url(url) if store_pub_final_results else url
+
         eth_addrs = list()
         hmt_amounts = list()
 
@@ -620,6 +627,7 @@ class Job:
             hmt_amounts.append(int(amount * 10 ** 18))
 
         func_args = [eth_addrs, hmt_amounts, url, hash_, 1]
+
         try:
             handle_transaction_with_retry(txn_func, self.retry, *func_args, **txn_info)
             return self._bulk_paid() is True
@@ -1088,7 +1096,7 @@ class Job:
         """
         return download(self.intermediate_manifest_url, priv_key)
 
-    def final_results(self, priv_key: bytes) -> Dict:
+    def final_results(self, priv_key: bytes) -> Optional[Dict]:
         """Retrieves the final results stored by the Reputation Oracle.
 
         >>> from test.hmt_escrow.utils import manifest
@@ -1122,7 +1130,13 @@ class Job:
         final_results_url = self.job_contract.functions.finalResultsUrl().call(
             {"from": self.gas_payer, "gas": Wei(self.gas)}
         )
-        return download(final_results_url, priv_key)
+
+        if not final_results_url:
+            return None
+
+        url = get_key_from_url(final_results_url)
+
+        return download(url, priv_key)
 
     def _access_job(self, factory_addr: str, escrow_addr: str, **credentials):
         """Given a factory and escrow address and credentials, access an already
